@@ -2,27 +2,30 @@ package com.fasterxml.jackson.core.filter;
 
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.Reader;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.filter.TokenFilter.Inclusion;
 import com.fasterxml.jackson.core.util.JsonGeneratorDelegate;
 
 /**
  * Specialized {@link JsonGeneratorDelegate} that allows use of
  * {@link TokenFilter} for outputting a subset of content that
  * caller tries to generate.
- * 
+ *
  * @since 2.6
  */
 public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 {
+
     /*
     /**********************************************************
     /* Configuration
     /**********************************************************
      */
-    
+
     /**
      * Object consulted to determine whether to write parts of content generator
      * is asked to write or not.
@@ -44,18 +47,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
      * done and only explicitly included entries are output; if `true` then
      * path from main level down to match is also included as necessary.
      */
-    protected boolean _includePath;
-
-    /* NOTE: this feature is included in the first version (2.6), but
-     * there is no public API to enable it, yet, since there isn't an
-     * actual use case. But it seemed possible need could arise, which
-     * is feature has not yet been removed. If no use is found within
-     * first version or two, just remove.
-     * 
-     * Marked as deprecated since its status is uncertain.
-     */
-    @Deprecated
-    protected boolean _includeImmediateParent;
+    protected TokenFilter.Inclusion _inclusion;
 
     /*
     /**********************************************************
@@ -76,7 +68,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
      * property, and also used for array elements.
      */
     protected TokenFilter _itemFilter;
-    
+
     /**
      * Number of tokens for which {@link TokenFilter#INCLUDE_ALL}
      * has been returned
@@ -89,8 +81,32 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     /**********************************************************
      */
 
+    /**
+     * @param d Generator to delegate calls to
+     * @param f Filter to use
+     * @param includePath Whether to include path or not
+     * @param allowMultipleMatches Whether to allow multiple matches
+     *
+     * @deprecated since 2.12 Use the constructor that takes {@link TokenFilter.Inclusion}
+     *    argument instead.
+     */
+    @Deprecated
     public FilteringGeneratorDelegate(JsonGenerator d, TokenFilter f,
             boolean includePath, boolean allowMultipleMatches)
+    {
+        this(d, f, includePath ? Inclusion.INCLUDE_ALL_AND_PATH : Inclusion.ONLY_INCLUDE_ALL, allowMultipleMatches);
+    }
+
+    /**
+     * @param d Generator to delegate calls to
+     * @param f Filter to use
+     * @param inclusion Definition of inclusion criteria
+     * @param allowMultipleMatches Whether to allow multiple matches
+     *
+     * @since 2.12
+     */
+    public FilteringGeneratorDelegate(JsonGenerator d, TokenFilter f,
+            TokenFilter.Inclusion inclusion, boolean allowMultipleMatches)
     {
         // By default, do NOT delegate copy methods
         super(d, false);
@@ -98,7 +114,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
         // and this is the currently active filter for root values
         _itemFilter = f;
         _filterContext = TokenFilterContext.createRootContext(f);
-        _includePath = includePath;
+        _inclusion = inclusion;
         _allowMultipleMatches = allowMultipleMatches;
     }
 
@@ -113,10 +129,12 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     public JsonStreamContext getFilterContext() {
         return _filterContext;
     }
-    
+
     /**
      * Accessor for finding number of matches, where specific token and sub-tree
      * starting (if structured type) are passed.
+     *
+     * @return Number of matches
      */
     public int getMatchCount() {
         return _matchCount;
@@ -127,7 +145,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     /* Public API, accessors
     /**********************************************************
      */
-    
+
     @Override
     public JsonStreamContext getOutputContext() {
         /* 11-Apr-2015, tatu: Choice is between pre- and post-filter context;
@@ -136,13 +154,13 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
          */
         return _filterContext;
     }
-    
+
     /*
     /**********************************************************
     /* Public API, write methods, structural
     /**********************************************************
      */
-    
+
     @Override
     public void writeStartArray() throws IOException
     {
@@ -169,11 +187,16 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
             delegate.writeStartArray();
+        } else if (_itemFilter != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray();
         } else {
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
         }
     }
 
+    @SuppressWarnings("deprecation")
     @Override
     public void writeStartArray(int size) throws IOException
     {
@@ -198,6 +221,10 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
             delegate.writeStartArray(size);
+        } else if (_itemFilter != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(size);
         } else {
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
         }
@@ -227,6 +254,10 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
             delegate.writeStartArray(forValue);
+        } else if (_itemFilter != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(forValue);
         } else {
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
         }
@@ -256,11 +287,15 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
             delegate.writeStartArray(forValue, size);
+        } else if (_itemFilter != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(forValue, size);
         } else {
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
         }
     }
-    
+
     @Override
     public void writeEndArray() throws IOException
     {
@@ -286,9 +321,10 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 
         TokenFilter f = _filterContext.checkValue(_itemFilter);
         if (f == null) {
+            _filterContext = _filterContext.createChildObjectContext(null, false);
             return;
         }
-        
+
         if (f != TokenFilter.INCLUDE_ALL) {
             f = f.filterStartObject();
         }
@@ -296,6 +332,10 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildObjectContext(f, true);
             delegate.writeStartObject();
+        } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildObjectContext(f, true);
+            delegate.writeStartObject();
         } else { // filter out
             _filterContext = _filterContext.createChildObjectContext(f, false);
         }
@@ -316,6 +356,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 
         TokenFilter f = _filterContext.checkValue(_itemFilter);
         if (f == null) {
+            _filterContext = _filterContext.createChildObjectContext(null, false);
             return;
         }
 
@@ -326,6 +367,10 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildObjectContext(f, true);
             delegate.writeStartObject(forValue);
+        } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildObjectContext(f, true);
+            delegate.writeStartObject(forValue);
         } else { // filter out
             _filterContext = _filterContext.createChildObjectContext(f, false);
         }
@@ -346,6 +391,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 
         TokenFilter f = _filterContext.checkValue(_itemFilter);
         if (f == null) {
+            _filterContext = _filterContext.createChildObjectContext(null, false);
             return;
         }
 
@@ -439,7 +485,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeString(value);
     }
 
@@ -461,7 +507,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeString(text, offset, len);
     }
 
@@ -482,10 +528,32 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeString(value);
     }
 
+    @Override
+    public void writeString(Reader reader, int len) throws IOException {
+        if (_itemFilter == null) {
+            return;
+        }
+        if (_itemFilter != TokenFilter.INCLUDE_ALL) {
+            TokenFilter state = _filterContext.checkValue(_itemFilter);
+            if (state == null) {
+                return;
+            }
+            if (state != TokenFilter.INCLUDE_ALL) {
+                // [core#609]: do need to implement, but with 2.10.x TokenFilter no
+                // useful method to call so will be mostly unfiltered
+                if (!state.includeString(reader, len)) {
+                    return;
+                }
+            }
+            _checkParentPath();
+        }
+        delegate.writeString(reader, len);
+    }
+
     @Override
     public void writeRawUTF8String(byte[] text, int offset, int length) throws IOException
     {
@@ -613,7 +681,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -634,7 +702,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -655,7 +723,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -676,7 +744,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -697,7 +765,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -718,7 +786,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -739,7 +807,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -764,6 +832,27 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
         delegate.writeNumber(encodedValue);
     }
 
+    @Override
+    public void writeNumber(char[] encodedValueBuffer, int offset, int length) throws IOException, UnsupportedOperationException
+    {
+        if (_itemFilter == null) {
+            return;
+        }
+        if (_itemFilter != TokenFilter.INCLUDE_ALL) {
+            TokenFilter state = _filterContext.checkValue(_itemFilter);
+            if (state == null) {
+                return;
+            }
+            if (state != TokenFilter.INCLUDE_ALL) {
+                if (!state.includeRawValue()) { // close enough?
+                    return;
+                }
+            }
+            _checkParentPath();
+        }
+        delegate.writeNumber(encodedValueBuffer, offset, length);
+    }
+
     @Override
     public void writeBoolean(boolean v) throws IOException
     {
@@ -781,7 +870,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeBoolean(v);
     }
 
@@ -802,7 +891,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNull();
     }
 
@@ -819,7 +908,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             delegate.writeOmittedField(fieldName);
         }
     }
-    
+
     /*
     /**********************************************************
     /* Public API, write methods, Native Ids
@@ -828,7 +917,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 
     // 25-Mar-2015, tatu: These are tricky as they sort of predate actual filtering calls.
     //   Let's try to use current state as a clue at least...
-    
+
     @Override
     public void writeObjectId(Object id) throws IOException {
         if (_itemFilter != null) {
@@ -842,7 +931,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             delegate.writeObjectRef(id);
         }
     }
-    
+
     @Override
     public void writeTypeId(Object id) throws IOException {
         if (_itemFilter != null) {
@@ -865,7 +954,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             delegate.writeObject(pojo);
             return;
         }
-        // NOTE: copied from 
+        // NOTE: copied from
         if (pojo == null) {
             writeNull();
         } else {
@@ -876,7 +965,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _writeSimpleObject(pojo);
         }
     }
-    
+
     @Override
     public void writeTree(TreeNode rootNode) throws IOException {
         if (delegateCopyMethods) {
@@ -925,13 +1014,23 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 
     protected void _checkParentPath() throws IOException
     {
-        ++_matchCount;
+        _checkParentPath(true);
+    }
+
+    protected void  _checkParentPath(boolean isMatch) throws IOException
+    {
+        if (isMatch) {
+            ++_matchCount;
+        }
         // only need to construct path if parent wasn't written
-        if (_includePath) {
+        if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
             _filterContext.writePath(delegate);
+        } else if (_inclusion == Inclusion.INCLUDE_NON_NULL) {
+            // path has already been written, except for maybe field name
+            _filterContext.ensureFieldNameWritten(delegate);
         }
         // also: if no multiple matches desired, short-cut checks
-        if (!_allowMultipleMatches) {
+        if (isMatch && !_allowMultipleMatches) {
             // Mark parents as "skip" so that further check calls are not made
             _filterContext.skipParentChecks();
         }
@@ -941,16 +1040,17 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
      * Specialized variant of {@link #_checkParentPath} used when checking
      * parent for a property name to be included with value: rules are slightly
      * different.
+     *
+     * @throws IOException If there is an issue with possible resulting read
      */
     protected void _checkPropertyParentPath() throws IOException
     {
         ++_matchCount;
-        if (_includePath) {
+        if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
             _filterContext.writePath(delegate);
-        } else if (_includeImmediateParent) {
-            // 21-Apr-2015, tatu: Note that there is no API to enable this currently...
-            //    retained for speculative future use
-            _filterContext.writeImmediatePath(delegate);
+        } else if (_inclusion == Inclusion.INCLUDE_NON_NULL) {
+            // path has already been written, except for maybe field name
+            _filterContext.ensureFieldNameWritten(delegate);
         }
 
         // also: if no multiple matches desired, short-cut checks
@@ -959,7 +1059,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _filterContext.skipParentChecks();
         }
     }
-    
+
     protected boolean _checkBinaryWrite() throws IOException
     {
         if (_itemFilter == null) {
@@ -974,7 +1074,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
         }
         return false;
     }
-    
+
     protected boolean _checkRawValueWrite() throws IOException
     {
         if (_itemFilter == null) {
