@@ -7,9 +7,9 @@ import java.io.StringReader;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
-import com.fasterxml.jackson.core.*;
-
 import org.junit.jupiter.api.Test;
+
+import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.exc.InputCoercionException;
 import com.fasterxml.jackson.core.io.NumberInput;
 import com.fasterxml.jackson.core.json.JsonReadFeature;
@@ -22,15 +22,18 @@ import static org.junit.jupiter.api.Assertions.*;
  */
 @SuppressWarnings("resource")
 class NumberParsingTest
-        extends JUnit5TestBase
+    extends JUnit5TestBase
 {
     protected JsonFactory jsonFactory() {
         return sharedStreamFactory();
     }
 
+    private final String ISSUE_4694_VALUE =
+        "-11000.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
+
     /*
     /**********************************************************************
-    /* Tests, Boolean
+    /* Tests, Boolean (why here?)
     /**********************************************************************
      */
 
@@ -451,7 +454,7 @@ class NumberParsingTest
         BigInteger biggie = new BigInteger(NUMBER_STR);
 
         for (int mode : ALL_MODES) {
-            try (JsonParser p = createParser(jsonFactory(), mode, NUMBER_STR +" ")) {
+            try (JsonParser p = createParser(jsonFactory(), mode, NUMBER_STR + " ")) {
                 assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
                 assertEquals(JsonParser.NumberType.BIG_INTEGER, p.getNumberType());
                 assertEquals(NUMBER_STR, p.getText());
@@ -460,6 +463,26 @@ class NumberParsingTest
         }
     }
 
+    @Test
+    void intsWith19Chars() throws Exception
+    {
+        final String[] values = new String[] {
+            "9223372036854775808", "9999999999999999999"
+        };
+        for (String value : values) {
+            BigInteger biggie = new BigInteger(value);
+
+            for (int mode : ALL_MODES) {
+                try (JsonParser p = createParser(jsonFactory(), mode, value + " ")) {
+                    assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+                    assertEquals(JsonParser.NumberType.BIG_INTEGER, p.getNumberType());
+                    assertEquals(biggie, p.getBigIntegerValue());
+                    assertEquals(value, p.getText());
+                }
+            }
+        }
+    }
+
     // Related to [core#1135]: JsonParser.isNaN() should not be fooled
     // by possible Double overflow
     @Test
@@ -472,7 +495,7 @@ class NumberParsingTest
         String tooBigString = tooBig.toPlainString();
 
         // sanity check that it is beyond Double range:
-        assertTrue(new Double(tooBigString).isInfinite());
+        assertTrue(Double.valueOf(tooBigString).isInfinite());
 
         for (int mode : ALL_MODES) {
             try (JsonParser p = createParser(jsonFactory(), mode, tooBigString +" ")) {
@@ -996,6 +1019,41 @@ class NumberParsingTest
         }
     }
 
+    // https://github.com/FasterXML/jackson-databind/issues/4694
+    @Test
+    void databind4694() throws Exception {
+        final BigDecimal expected = new BigDecimal(ISSUE_4694_VALUE);
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(mode, String.format(" %s ", ISSUE_4694_VALUE))) {
+                assertEquals(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(expected, p.getDecimalValue());
+                assertFalse(p.isNaN());
+            }
+        }
+    }
+
+    void databind4694Double() throws Exception {
+        final Double expected = Double.valueOf(ISSUE_4694_VALUE);
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(mode, String.format(" %s ", ISSUE_4694_VALUE))) {
+                assertEquals(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(expected, p.getDoubleValue());
+                assertFalse(p.isNaN());
+            }
+        }
+    }
+
+    void databind4694Float() throws Exception {
+        final Float expected = Float.valueOf(ISSUE_4694_VALUE);
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(mode, String.format(" %s ", ISSUE_4694_VALUE))) {
+                assertEquals(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(expected, p.getFloatValue());
+                assertFalse(p.isNaN());
+            }
+        }
+    }
+
     /*
     /**********************************************************
     /* Helper methods
